MODULE Simulation; (* Lino Guzzella / 02.09.85 / ETHZ Abt 3A *)
                   (* last modified  04.09.85 fxk *)
(* Simulates a system of n ordinal differential equations *)
       
FROM MathLib0  IMPORT sin,cos,sqrt,entier;
FROM MathLib1 IMPORT random;
FROM Terminal IMPORT WriteString,Read,GotoXY,ClearTerminal;
FROM RealTerminal IMPORT ReadR,WriteR,RFormat;
FROM OutTerminal IMPORT WriteI;
FROM QuickDraw IMPORT MoveTo,LineTo,GetPort,SetPort,GrafPtr,SetRect;
FROM SYSTEM IMPORT ADR;
FROM OStrings IMPORT mkRef1;
FROM fxWindows IMPORT WindowPtr,allocateWindow,deallocateWindow,defaultSetup,
                      wDefRec,wDefPtr;
FROM MemoryMgr IMPORT NewPtr;
FROM DeviceMgr IMPORT DriverOpen,DriverClose,DriverControl,OpParamType,
                      OpParamPtr;
FROM OStrings  IMPORT mkOStr,OStr15;

CONST
  n = 5;
  m = 2;

TYPE
  Vector1 = ARRAY[1..n] OF REAL;
  Vector2 = ARRAY[1..m] OF REAL;
 
  
VAR
  x,xalt : Vector1;
  s,salt : Vector2;
  t,talt,tfinal,delta,d1,d2 : REAL;
  i,j,graphx,graphy,refnumber,error : INTEGER;
  p : OpParamPtr;
  PtoPasStr : POINTER TO OStr15;
  ch : CHAR;
  savePort:GrafPtr;
  SimulationWindow:WindowPtr; Setup:wDefPtr; SetupVar:wDefRec;

PROCEDURE fun (VAR fx:Vector1;VAR s:Vector2;x:Vector1;t,d1,d2:REAL;n:INTEGER);

VAR 
  r : ARRAY[1..4] OF REAL;
  u1,u2,s1,s2,xb : REAL;
  i : INTEGER;

BEGIN
  s[1]:= x[1] + 2.0 * x[2] + x[3];
  s[2]:= x[4] + x[5];
  xb:= sqrt(x[1]*x[1] + x[2]*x[2]+ x[3]*x[3]+ x[4]*x[4]+ x[5]*x[5]);
 
  IF s[1] > 0.0 THEN u1:= x[1] + x[2] + x[3] - x[4] - d1 * xb 
              ELSE u1:= x[1] + x[2] + x[3] - x[4] + d1 * xb 
  END;

  IF s[2] > 0.0 THEN u2:= -x[1] - x[3] + 2.0 * x[4] - 2.0 * x[5] - d2 * xb 
              ELSE u2:= -x[1] - x[3] + 2.0 * x[4] - 2.0 * x[5] + d2 * xb 
  END;
  
  r[1]:= cos(14.0*t);
  r[2]:= -r[1];
  r[3]:= 0.5*cos(19.0*t);
  r[4]:= -0.5*r[3];

  fx[1] := x[2];
  fx[2] := x[3];
  fx[3] := (-1.0+r[3])*x[1]-2.0*x[2]+(r[1]-3.0)*x[3]+(1.0+r[4])*x[4]+r[2]*x[5]+(1.0+r[3])*u1+r[4]*u2;
  fx[4] := x[5];
  fx[5] := (1.0+r[2])*x[1]+(1.0+r[3])*x[3]+(r[4]-2.0)*x[4]+(1.0+r[1])*x[5]+(1.0+r[3])*u2;

END fun;


PROCEDURE step(VAR x:Vector1;VAR s:Vector2;delta,t,d1,d2:REAL;n:INTEGER);
VAR
  xaux,dx1,dx2,dx3,dx4:Vector1;
  tlocal : REAL;
  i : INTEGER;
BEGIN
  tlocal:= t;
  FOR i:= 1 TO n DO xaux[i]:= x[i] END;
  fun(dx1,s,xaux,tlocal,d1,d2,n);
  tlocal:= t + 0.5 * delta;
  FOR i:= 1 TO n DO xaux[i]:= x[i] + 0.5 * delta * dx1[i] END;
  fun(dx2,s,xaux,tlocal,d1,d2,n);
  FOR i:= 1 TO n DO xaux[i]:= x[i] + 0.5 * delta * dx2[i] END;
  fun(dx3,s,xaux,tlocal,d1,d2,n);
  tlocal:= t + delta;
  FOR i:= 1 TO n DO xaux[i]:= x[i] + delta * dx3[i] END;
  fun(dx4,s,xaux,tlocal,d1,d2,n);
  FOR i:=1 TO n DO x[i]:=x[i] + delta*(dx1[i]+2.0 * dx2[i]+2.0 * dx3[i]+dx4[i])/6.0 END;  
END step;

BEGIN (* body of main program *)
 GetPort(savePort);

 Setup:=ADR(SetupVar); defaultSetup(Setup);
 WITH Setup^ DO title:=mkRef1("Simulation");
                SetRect(boundsRect,4,40,508,338); 
             END;
 allocateWindow(SimulationWindow,Setup,NIL);
 SetPort(SimulationWindow); 

ch:= 'y';
  REPEAT
    x[1]:= 1.0;
    x[2]:= 0.0;
    x[3]:= 0.0;
    x[4]:= 1.0;
    x[5]:= 0.0;
    t:= 0.0;
    talt:= 0.0;
    FOR i:= 1 TO n DO xalt[i]:= x[i] END;
    MoveTo(200,10);
    WriteString('Regulator Gain d1 = ? ');
    ReadR(d1);
    MoveTo(200,30);
    WriteString('Regulator Gain d2 = ? ');
    ReadR(d2);
    MoveTo(200,50);
    WriteString('Integration Step delta = ? ');
    ReadR(delta);
    MoveTo(200,70);
    WriteString('Final Time tfinal = ? ');
    ReadR(tfinal);
    MoveTo(0,150);
    LineTo(600,150);
    MoveTo(50,0);
    LineTo(50,500);
    MoveTo(0,250);
    LineTo(600,250);
    MoveTo(0,300);
    LineTo(600,300);
  
    REPEAT
      step(x,s,delta,t,d1,d2,n);
      t:= t + delta;
      FOR j:= 1 TO n DO
        graphx:= entier(50.0 * talt+50.0);
        graphy:= entier(100.0 *(1.5-xalt[j]));
        MoveTo(graphx,graphy);
        graphx:= entier(50.0 * t+50.0);
        graphy:= entier(100.0 * (1.5-x[j]));
        LineTo(graphx,graphy);
        xalt[j]:= x[j];
      END;
      IF(s[1]<0.05)AND(s[1]>-0.05)AND(salt[1]<0.05)AND(salt[1]>-0.05)THEN
        graphx:= entier(50.0 * talt+50.0);
        graphy:= entier(1000.0*(0.25-salt[1]));
        MoveTo(graphx,graphy);
        graphx:= entier(50.0*t+50.0);
        graphy:= entier(1000.0*(0.25-s[1]));
        LineTo(graphx,graphy);
      END;
      IF(s[2]<0.05)AND(s[2]>-0.05)AND(salt[2]<0.05)AND(salt[2]>-0.05)THEN
        graphx:= entier(50.0 * talt+50.0);
        graphy:= entier(1000.0*(0.3-salt[2]));
        MoveTo(graphx,graphy);
        graphx:= entier(50.0*t+50.0);
        graphy:= entier(1000.0*(0.3-s[2]));
        LineTo(graphx,graphy);
      END;
	salt[1]:= s[1];
	salt[2]:= s[2];
      talt:= t; 
    UNTIL t >= tfinal;
    MoveTo(350,250);
    WriteString(' continue (y) or print(p)  ?');
    Read(ch);
    IF ch = 'p' THEN
      PtoPasStr:= NewPtr(15);
      mkOStr('.Print',PtoPasStr^);
      error:=DriverOpen(PtoPasStr,refnumber);
      MoveTo(350,270);
      (*  WriteString('refnumber');
      WriteI(refnumber,5);*)
      p:= NewPtr(100);
      p^.param1:=196606;
      error:=DriverControl(refnumber,6,p);
      error:=DriverClose(refnumber);
    END;
    ClearTerminal;
  UNTIL ch<>'y';

 deallocateWindow(SimulationWindow); 
 SetPort(savePort);
END Simulation.
